@@ -1,15 +1,15 @@
 package Image::TIFF;
 
 # Copyright 1999-2001, Gisle Aas.
-# Copyright 2006 Tels
+# Copyright 2006, 2007 Tels
 #
 # This library is free software; you can redistribute it and/or
-# modify it under the same terms as Perl itself.
+# modify it under the same terms as Perl v5.8.8 itself.
 
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '1.03';
+$VERSION = '1.06';
 
 my @types = (
   [ "BYTE",      "C1", 1],
@@ -19,11 +19,14 @@ my @types = (
   [ "RATIONAL",  "N2", 8],
   [ "SBYTE",     "c1", 1],
   [ "UNDEFINED", "a1", 1],
+  [ "BINARY",    "a1", 1],  # treat binary data as UNDEFINED	
   [ "SSHORT",    "n1", 2],
   [ "SLONG",     "N1", 4],
   [ "SRATIONAL", "N2", 8],
   [ "FLOAT",     "f1", 4],  # XXX 4-byte IEEE format
   [ "DOUBLE",    "d1", 8],  # XXX 8-byte IEEE format
+# XXX TODO:
+#  [ "IFD",      "??", ?],  # See ExifTool
 );
 
 my %nikon1_tags = (
@@ -327,6 +330,7 @@ my %panasonic_tags = (
 	1 => 'Auto',
 	2 => 'Manual',
 	5 => 'Auto, Continuous',
+	4 => 'Auto, Focus button',
 	},
     0x000f => { __TAG__ => "SpotMode",
 	# XXX TODO: does not decode properly
@@ -378,8 +382,12 @@ my %panasonic_tags = (
 	5 => 'Sepia',
 	},
     0x0029 => "Panasonic 0x0029",
-    0x002a => "Panasonic 0x002a",
-    0x002b => "Panasonic 0x002b",
+    0x002a => { __TAG__ => "BurstMode",
+	0 => 'Off',
+	1 => 'Low/High Quality',
+	2 => 'Infinite',
+	},
+    0x002b => "ImageSequenceNumber",
     0x002c => { __TAG__ => "Contrast",
 	0 => 'Normal',
 	1 => 'Low',
@@ -393,7 +401,11 @@ my %panasonic_tags = (
 	1 => 'Low',
 	2 => 'High',
 	},
-    0x002e => "Panasonic 0x002e",
+    0x002e => { __TAG__ => "SelfTimer",
+	1 => 'Off',
+	2 => '10s',
+	3 => '2s',
+	},
     0x002f => "Panasonic 0x002f",
     0x0030 => "Panasonic 0x0030",
     0x0031 => "Panasonic 0x0031",
@@ -690,13 +702,25 @@ my %tiff_tags = (
   257   => "height",
   258   => "BitsPerSample",
   259   => { __TAG__ => "Compression",
-	     1 => "PackBytes",
-	     2 => "CCITT Group3",
-	     3 => "CCITT T4",
-	     4 => "CCITT T6",
-	     5 => "LZW",
-	     6 => "JPEG",
-	     32773 => "PackBits",
+		1 => "PackBytes",
+		2 => "CCITT Group3",
+		3 => "CCITT T4",
+		4 => "CCITT T6",
+		5 => "LZW",
+		6 => "JPEG",
+		7 => "JPEG DCT",
+		8 => "Deflate (Adobe)",
+		32766 => "NeXT 2-bit RLE",
+		32771 => "#1 w/ word alignment",
+		32773 => "PackBits (Macintosh RLE)",
+		32809 => "ThunderScan RLE",
+		32895 => "IT8 CT w/padding",
+		32896 => "IT8 Linework RLE",
+		32897 => "IT8 Monochrome picture",
+		32898 => "IT8 Binary line art",
+		32908 => "Pixar companded 10bit LZW",
+		32909 => "Pixar companded 11bit ZIP",
+		32946 => "Deflate",
            },
   262   => { __TAG__ => "PhotometricInterpretation",
 	     0 => "WhiteIsZero",
@@ -717,6 +741,7 @@ my %tiff_tags = (
 	     1 => "LowInHigh",
 	     2 => "HighInLow",
 	   },
+  269   => "DocumentName",
   270   => "ImageDescription",
   271   => "Make",
   272   => "Model",
@@ -796,7 +821,15 @@ sub new
 	while ($ifd) {
 	    push(@{$self->{ifd}}, $ifd);
 	    my($num_fields) = $self->unpack("x$ifd n", $_);
-	    $ifd = $self->unpack("N", substr($_, $ifd + 2 + $num_fields*12, 4));
+	    my $next_ifd = $self->unpack("N", substr($_, $ifd + 2 + $num_fields*12, 4));
+	    # guard against (bug #26127)
+	    $next_ifd = 0 if $next_ifd > length($_);
+	    # guard against looping ifd (bug #26130)
+	    if ($next_ifd <= $ifd) {
+		# bad TIFF header - would cause a loop or strange results
+	        last;
+            }
+        $ifd = $next_ifd;
 	}
     }
 
@@ -843,6 +876,8 @@ sub add_fields
 {
     my($self, $offset, $ifds, $tags, $voff_plus) = @_;
     return unless $offset;
+    # guard against finding the same offset twice (bug #29088)
+    return if $self->{seen_offset}{$offset}++;
     $tags ||= \%tiff_tags;
 
     for (${$self->{source}}) {  # alias as $_
@@ -871,7 +906,8 @@ sub add_fields
 	    # extract type information
 	    my($tmpl, $vlen);
 	    ($type, $tmpl, $vlen) = @{$types[$type-1]};
-	    die "Assert" unless $type;
+
+	    die "Undefined type while parsing" unless $type;
 
 	    if ($count * $vlen <= 4) {
 		$voff = $entry_offset + 8;
@@ -884,6 +920,7 @@ sub add_fields
 	    else {
 		$voff += $voff_plus || 0;
 	    }
+
 	    $tmpl =~ s/(\d+)$/$count*$1/e;
 
 	    my @v = $self->unpack("x$voff $tmpl", $_);
@@ -896,12 +933,24 @@ sub add_fields
 	    my $val = (@v > 1) ? \@v : $v[0];
 	    bless $val, "Image::TIFF::Rational" if $type =~ /^S?RATIONAL$/;
 
-	    # avoid things like "Foo\x00\x00\x00" by removing trailing nulls
 	    if ($type eq 'ASCII' || $type eq 'UNDEFINED')
-	      {
-	      $val =~ /^([^\x00]+)/;
-	      $val = '' . ($1 || '');
-	      }
+		{
+		if ($val =~ /\0[^\0]+\z/)
+		    {
+		    # cut out the first "ASCII\0" and take the remaining text
+		    # (fix bug #29243 parsing of UserComment)
+		    # XXX TODO: this needs to handle UNICODE, too:
+		    $val =~ /\0([^\0]+)/;
+		    $val = '' . ($1 || '');
+		    }
+		else
+		    {
+		    # avoid things like "Foo\x00\x00\x00" by removing trailing nulls
+		    # this needs to handle UNICODE, too:
+		    $val =~ /^([^\0]*)/;
+		    $val = '' . ($1 || '');
+		    }
+		}
 
 	    $tag = $tags->{$tag} || $self->tagname($tag);
 
